@@ -979,6 +979,7 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
       shared.Settings.ALIASING_FUNCTION_POINTERS = 0
 
     if shared.Settings.SPLIT_MEMORY:
+      assert shared.Settings.SPLIT_MEMORY > shared.Settings.TOTAL_STACK, 'SPLIT_MEMORY must be at least TOTAL_STACK (stack must fit in first chunk)'
       assert shared.Settings.SPLIT_MEMORY & (shared.Settings.SPLIT_MEMORY-1) == 0, 'SPLIT_MEMORY must be a power of 2'
       if shared.Settings.ASM_JS == 1:
         logging.warning('not all asm.js optimizations are possible with SPLIT_MEMORY, disabling those')
@@ -1045,6 +1046,12 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
     if proxy_to_worker:
       shared.Settings.PROXY_TO_WORKER = 1
 
+    if use_preload_plugins:
+      shared.Settings.FORCE_FILESYSTEM = 1 # preload plugins require preload support which is part of the filesystem
+
+    if proxy_to_worker or use_preload_plugins:
+      shared.Settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE += ['$Browser']
+
     if js_opts:
       shared.Settings.RUNNING_JS_OPTS = 1
 
@@ -1052,12 +1059,14 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
       shared.Settings.EXPORTED_FUNCTIONS += ['___errno_location'] # so FS can report errno back to C
       if not shared.Settings.NO_EXIT_RUNTIME:
         shared.Settings.EXPORTED_FUNCTIONS += ['_fflush'] # to flush the streams on FS quit
+                                                          # TODO this forces 4 syscalls, maybe we should avoid it?
 
     if shared.Settings.USE_PTHREADS:
       if not any(s.startswith('PTHREAD_POOL_SIZE=') for s in settings_changes):
         settings_changes.append('PTHREAD_POOL_SIZE=0')
       js_libraries.append(shared.path_from_root('src', 'library_pthread.js'))
       newargs.append('-D__EMSCRIPTEN_PTHREADS__=1')
+      shared.Settings.FORCE_FILESYSTEM = 1 # proxying of utime requires the filesystem
     else:
       js_libraries.append(shared.path_from_root('src', 'library_pthread_stub.js'))
 
@@ -1080,6 +1089,10 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
       shared.Settings.GLOBAL_BASE = 1024 # leave some room for mapping global vars
       assert not shared.Settings.SPLIT_MEMORY, 'WebAssembly does not support split memory'
 
+    if tracing:
+      if shared.Settings.ALLOW_MEMORY_GROWTH:
+        shared.Settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE += ['emscripten_trace_report_memory_layout']
+
     if shared.Settings.ONLY_MY_CODE:
       shared.Settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = []
       separate_asm = True
