@@ -261,6 +261,17 @@ var LibraryPThreadStub = {
     {{{ makeSetValue('ptr', 4, 'h^valh', 'i32') }}};
     {{{ makeStructuralReturn(['l', 'h']) }}};
   },
+
+  emscripten_atomic_add_u32: 'llvm_atomic_load_add_i32_p0i32',
+  emscripten_atomic_load_u64: '__atomic_load_8',
+  emscripten_atomic_store_u64: '__atomic_store_8',
+  emscripten_atomic_cas_u64: '__atomic_compare_exchange_8',
+  emscripten_atomic_exchange_u64: '__atomic_exchange_8',
+  _emscripten_atomic_fetch_and_add_u64: '__atomic_fetch_add_8',
+  _emscripten_atomic_fetch_and_sub_u64: '__atomic_fetch_sub_8',
+  _emscripten_atomic_fetch_and_and_u64: '__atomic_fetch_and_8',
+  _emscripten_atomic_fetch_and_or_u64: '__atomic_fetch_or_8',
+  _emscripten_atomic_fetch_and_xor_u64: '__atomic_fetch_xor_8',
 };
 
 mergeInto(LibraryManager.library, LibraryPThreadStub);
