@@ -174,6 +174,16 @@ namespace emscripten {
                 GenericFunction invoker,
                 GenericFunction method);
 
+            void _embind_register_class_class_property(
+                TYPEID classType,
+                const char* fieldName,
+                TYPEID fieldType,
+                const void* fieldContext,
+                const char* getterSignature,
+                GenericFunction getter,
+                const char* setterSignature,
+                GenericFunction setter);
+
             EM_VAL _embind_create_inheriting_constructor(
                 const char* constructorName,
                 TYPEID wrapperType,
@@ -522,6 +532,20 @@ namespace emscripten {
             }
         };
 
+        template<typename FieldType>
+        struct GlobalAccess {
+            typedef internal::BindingType<FieldType> MemberBinding;
+            typedef typename MemberBinding::WireType WireType;
+
+            static WireType get(FieldType* context) {
+                return MemberBinding::toWireType(*context);
+            }
+
+            static void set(FieldType* context, WireType value) {
+                *context = MemberBinding::fromWireType(value);
+            }
+        };
+
         // TODO: This could do a reinterpret-cast if sizeof(T) === sizeof(void*)
         template<typename T>
         inline T* getContext(const T& t) {
@@ -1357,10 +1381,45 @@ namespace emscripten {
                 args.getCount(),
                 args.getTypes(),
                 getSignature(invoke),
-                reinterpret_cast<internal::GenericFunction>(invoke),
+                reinterpret_cast<GenericFunction>(invoke),
                 reinterpret_cast<GenericFunction>(classMethod));
             return *this;
         }
+
+        template<typename FieldType>
+        EMSCRIPTEN_ALWAYS_INLINE const class_& class_property(const char* name, const FieldType* field) const {
+            using namespace internal;
+
+            auto getter = &GlobalAccess<FieldType>::get;
+            _embind_register_class_class_property(
+                TypeID<ClassType>::get(),
+                name,
+                TypeID<FieldType>::get(),
+                field,
+                getSignature(getter),
+                reinterpret_cast<GenericFunction>(getter),
+                0,
+                0);
+            return *this;
+        }
+
+        template<typename FieldType>
+        EMSCRIPTEN_ALWAYS_INLINE const class_& class_property(const char* name, FieldType* field) const {
+            using namespace internal;
+
+            auto getter = &GlobalAccess<FieldType>::get;
+            auto setter = &GlobalAccess<FieldType>::set;
+            _embind_register_class_class_property(
+                TypeID<ClassType>::get(),
+                name,
+                TypeID<FieldType>::get(),
+                field,
+                getSignature(getter),
+                reinterpret_cast<GenericFunction>(getter),
+                getSignature(setter),
+                reinterpret_cast<GenericFunction>(setter));
+            return *this;
+        }
     };
 
     ////////////////////////////////////////////////////////////////////////////////
