@@ -1063,6 +1063,7 @@ class Building:
     env['CXX'] = EMXX if not WINDOWS else 'python %r' % EMXX
     env['AR'] = EMAR if not WINDOWS else 'python %r' % EMAR
     env['LD'] = EMCC if not WINDOWS else 'python %r' % EMCC
+    env['NM'] = LLVM_NM
     env['LDSHARED'] = EMCC if not WINDOWS else 'python %r' % EMCC
     env['RANLIB'] = EMRANLIB if not WINDOWS else 'python %r' % EMRANLIB
     env['EMMAKEN_COMPILER'] = Building.COMPILER
@@ -1086,24 +1087,24 @@ class Building:
     def is_exe(fpath):
       return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
 
+    if os.path.isabs(program):
+      if os.path.isfile(program): return program
+
+      if WINDOWS:
+        for suffix in ['.exe', '.cmd', '.bat']:
+          if is_exe(program + suffix): return program + suffix
+
     fpath, fname = os.path.split(program)
     if fpath:
-      if is_exe(program):
-        return program
+      if is_exe(program): return program
     else:
       for path in os.environ["PATH"].split(os.pathsep):
         path = path.strip('"')
         exe_file = os.path.join(path, program)
-        if is_exe(exe_file):
-          return exe_file
-
-        if WINDOWS and not '.' in fname:
-          if is_exe(exe_file + '.exe'):
-            return exe_file + '.exe'
-          if is_exe(exe_file + '.cmd'):
-            return exe_file + '.cmd'
-          if is_exe(exe_file + '.bat'):
-            return exe_file + '.bat'
+        if is_exe(exe_file): return exe_file
+        if WINDOWS:
+          for suffix in ['.exe', '.cmd', '.bat']:
+            if is_exe(exe_file + suffix): return exe_file + suffix
 
     return None
 
@@ -1438,16 +1439,10 @@ class Building:
 
       response_fh = open(response_file, 'w')
       for arg in actual_files:
-        # we can't put things with spaces in the response file
-        if " " in arg:
-          link_args.append(arg)
-        else:
-          response_fh.write(arg + "\n")
+        # escaped double quotes allows 'space' characters in pathname the response file can use
+        response_fh.write("\"" + arg + "\"\n")
       response_fh.close()
 
-      if len(' '.join(link_args)) > 8192:
-        logging.warning('emcc: link command line is very long, even with response file -- use paths with no spaces')
-
     if not just_calculate:
       logging.debug('emcc: llvm-linking: %s to %s', actual_files, target)
       output = Popen([LLVM_LINK] + link_args + ['-o', target], stdout=PIPE).communicate()[0]
@@ -1937,8 +1932,9 @@ class JS:
     s = ''.join(map(chr, s))
     s = s.replace('\\', '\\\\').replace("'", "\\'")
     s = s.replace('\n', '\\n').replace('\r', '\\r')
+    # Escape the ^Z (= 0x1a = substitute) ASCII character and all characters higher than 7-bit ASCII.
     def escape(x): return '\\x{:02x}'.format(ord(x.group()))
-    return re.sub('[\x80-\xff]', escape, s)
+    return re.sub('[\x1a\x80-\xff]', escape, s)
 
 def execute(cmd, *args, **kw):
   try:
